$:.push File.dirname(__FILE__)
require 'pry'
require 'logger'
require 'faraday'
require 'ostruct'
require 'faraday_middleware'
require 'faraday_middleware/multi_json'
require 'multi_xml'
require 'comma'
require 'active_support/core_ext/hash/indifferent_access'
require 'active_support/core_ext/object/blank'
require 'active_support/core_ext/module'
require 'active_support/inflector'
require 'active_support/notifications'
require 'routengn_client/version'
require 'routengn_client/exceptions'
require 'routengn_client/logging'
require 'routengn_client/utils'
require 'routengn_client/connection'
require 'routengn_client/request'
require 'routengn_client/response'
require 'routengn_client/model'
require 'routengn_client/remote_model'

module RouteNGNClient
  autoload :RouteNGNResponseMiddleware, 'routengn_client/middleware/routengn_response_middleware'
  autoload :Account, 'routengn_client/models/account'
  autoload :SystemAdminAccount, 'routengn_client/models/system_admin_account'
  autoload :User, 'routengn_client/models/user'
  autoload :OutboundEndPointGroup, 'routengn_client/models/outbound_end_point_group'
  autoload :InboundEndPointGroup, 'routengn_client/models/inbound_end_point_group'
  autoload :Carrier, 'routengn_client/models/carrier'
  autoload :Contact, 'routengn_client/models/contact'
  autoload :EndPoint, 'routengn_client/models/end_point'
  autoload :AllowList, 'routengn_client/models/allow_list'
  autoload :URIMatcher, 'routengn_client/models/uri_matcher'
  autoload :UriMatcher, 'routengn_client/models/uri_matcher'
  autoload :RouteTable, 'routengn_client/models/route_table'
  autoload :Route, 'routengn_client/models/route'
  autoload :RouteContainer, 'routengn_client/models/route_container'
  autoload :RateSheet, 'routengn_client/models/rate_sheet'
  autoload :InboundRate, 'routengn_client/models/inbound_rate'
  autoload :OutboundRate, 'routengn_client/models/outbound_rate'
  autoload :CodeSet, 'routengn_client/models/code_set'
  autoload :Code, 'routengn_client/models/code'
  autoload :CodeSheet, 'routengn_client/models/code_sheet'
  autoload :Record, 'routengn_client/models/record'

  module ClassMethods
    def connection
      @connection ? @connection : (raise RateNGNClientConnectionException)
    end

    def connect(config = {})
      @connection = Connection.new(:uri => config[:uri], :api_key => config[:api_key])
    end
  end

  extend ClassMethods
  extend Logging

  #MultiJson.use :yajl
  #Faraday.default_adapter = :excon
  Faraday.register_middleware :response, :routengn_response => lambda { RouteNGNResponseMiddleware }
end

ActiveSupport::Notifications.subscribe('request.faraday') do |name, start_time, end_time, _, env|
  url = env[:url]
  http_method = env[:method].to_s.upcase
  duration = end_time - start_time
  RouteNGNClient.logger.info '[%s] %s %s (%.3f s)' % [url.host, http_method, url.request_uri, duration]
end
