module RouteNGNClient
  class Contact < Model

    def route
      @attributes.route
    end

    def end_point
      @attributes.end_point
    end

    def inbound_rate
      self.route ? self.route.inbound_rate : nil
    end

    def outbound_rate
      self.route ? self.route.outbound_rate : nil
    end

    def outbound_end_point_group
      self.route.outbound_end_point_group if self.route
    end

    def aor
      @attributes.aor
    end

    def build_aor(sip_request, inbound_end_point_group, options = {})
      options = options.with_indifferent_access

      uri = self.build_uri(sip_request)

      template = Template.new(inbound_end_point_group.attributes.contact_template)
      template.sub!(self, uri, :inbound_end_point_host => options[:inbound_end_point_host], :inbound_end_point_port => options[:inbound_end_point_port])

      @attributes.aor = template.contact_template
    end

    def build_uri(sip_request, options = {})
      options = options.with_indifferent_access

      request_uri = sip_request.request_uri
      sip_uri = request_uri.clone
      oepg = self.outbound_end_point_group
      oepg_parts = oepg ? oepg.build_uri_parts : {}
      oep_parts = self.end_point ? self.end_point.build_uri_parts : {}

      dialcode = oepg && oepg.attributes.dialcode_expression ? sip_uri.user.dialcode(oepg.dialcode_regexp) : sip_uri.user.user
      dialcode = oepg.attributes.dialcode_template.gsub("${dialcode_match}", dialcode) if dialcode && oepg && oepg.attributes.dialcode_template

      blacklisted_user_params = oepg.attributes.uri_user_params_blacklist || [] if oepg
      blacklisted_params = oepg.attributes.uri_params_blacklist || [] if oepg
      merge_user_params, merge_params = {}, {}

      merge_user_params['rn'] = sip_request.rn if sip_request.rn && !sip_request.request_uri.params.keys.include?('rn') && !sip_request.request_uri.params.keys.include?('rn')
      merge_user_params['npdi'] = sip_request.npdi if sip_request.npdi && !sip_request.request_uri.params.keys.include?('npdi') && !sip_request.request_uri.params.keys.include?('npdi')
      merge_user_params.merge! oepg_parts[:uri_user_params]
      merge_params['rn'] = sip_request.rn if sip_request.rn && !merge_user_params.has_key?('rn')
      merge_params['npdi'] = sip_request.npdi if sip_request.npdi && !merge_user_params.has_key?('npdi')
      merge_params.merge! oepg_parts[:uri_params]

      sip_uri.regenerate(:user => {
                                   :user => dialcode,
                                   :merge_params => merge_user_params,
                                   :blacklisted_params => blacklisted_user_params
                                  },
                         :host => oep_parts[:host],
                         :port => oep_parts[:port],
                         :merge_params => merge_params,
                         :blacklisted_params => blacklisted_params)
    end

    class Template

      attr_accessor :contact_template

      def initialize(contact_template)
        @contact_template = contact_template.clone
      end

      def sub!(contact, sip_uri, options = {})
        options = options.with_indifferent_access

        iep_host = options[:inbound_end_point_host]
        iep_port = options[:inbound_end_point_port]

        #@contact_template.gsub!("${oepg.prefix}", '') # legacy as of 10/24/12
        @contact_template.gsub!("${ruri.dialcode}", sip_uri.user.user_string)
        @contact_template.gsub!("${ruri.user_params}", !sip_uri.user.params.blank? ? ";#{sip_uri.user.params_string}" : '')
        @contact_template.gsub!("${ruri.params}", !sip_uri.params.blank? ? ";#{sip_uri.params_string}" : '')
        @contact_template.gsub!("${iep.host}", !iep_host.blank? ? iep_host : '')
        @contact_template.gsub!("${iep.port}", !iep_port.blank? ? ":#{iep_port}" : '')
        @contact_template.gsub!("${oep.host}", sip_uri.host)
        @contact_template.gsub!("${oep.port}", ":#{sip_uri.port}")
        @contact_template.gsub!("${oepg.prefix}", '')  # legacy as of 10/24/12
        @contact_template.gsub!("${oepg.uri_user_params}", '') #legacy as of 3/29/13 (params already merged in)
        @contact_template.gsub!("${oepg.uri_params}", '') #legacy as of 3/29/13 (params already merged in)

        self.sub_outbound_rate! contact.outbound_rate
        self.sub_inbound_rate! contact.inbound_rate
      end

      def sub_outbound_rate!(rate)
        @contact_template.gsub!("${or.price}", rate && rate.attributes.price ? rate.attributes.price.to_s : '?')
        @contact_template.gsub!("${or.type}", rate && rate.attributes.type ? rate.attributes.type : '?')
        @contact_template.gsub!("${or.dialcode}", rate && rate.attributes.dialcode ? rate.attributes.dialcode : '?')
        @contact_template.gsub!("${or.lata}", rate && rate.attributes.lata ? rate.attributes.lata : '?')
        @contact_template.gsub!("${or.ocn}", rate && rate.attributes.ocn ? rate.attributes.ocn : '?')
        @contact_template.gsub!("${or.billing_increment}", rate && rate.attributes.billing_increment ? rate.attributes.billing_increment.to_s : '?')
      end

      def sub_inbound_rate!(rate)
        @contact_template.gsub!("${ir.price}", rate && rate.attributes.price ? rate.attributes.price.to_s : '?')
        @contact_template.gsub!("${ir.type}", rate && rate.attributes.type ? rate.attributes.type : '?')
        @contact_template.gsub!("${ir.dialcode}", rate && rate.attributes.dialcode ? rate.attributes.dialcode : '?')
        @contact_template.gsub!("${ir.billing_increment}", rate && rate.attributes.billing_increment ? rate.attributes.billing_increment.to_s : '?')
        @contact_template.gsub!("${ir.lata}", rate && rate.attributes.lata ? rate.attributes.lata : '?')
        @contact_template.gsub!("${ir.ocn}", rate && rate.attributes.ocn ? rate.attributes.ocn : '?')
      end

    end

  end
end