module RouteNGNClient
  class InboundEndPointGroup < RemoteModel

    set_resource_attributes({
                              :path_base => 'metallic/inbound_end_point_groups',
                              :collection_name => 'inbound_end_point_groups',
                              :per_page => 200,
                              :request_options => {
                              }
                            })

    belongs_to :carrier
    has_many :end_points, :inbound_rates, :rate_sheets
    has_one :allow_list

    attr_accessor :account

    def initialize(attributes = {})
      super

      @account ||= nil
    end

    def self.find_by_end_point_host(host, options = {}, &callback)
      options = { :per_page => 200 }.merge(options)
      all({:inbound_end_point_group => {:'end_points.host' => host}}, options, &callback)
    end

    def primary_route_table(options = {}, &callback)
      if self.attributes.primary_route_table_id.blank?
        callback.call(nil) if callback
      else
        RouteTable.find self.attributes.primary_route_table_id, options, &callback
      end
    end

    def secondary_route_table(options = {}, &callback)
      if self.attributes.secondary_route_table_id.blank?
        callback.call(nil) if callback
      else
        RouteTable.find self.attributes.secondary_route_table_id, options, &callback
      end
    end

    def dialcode_regexp
      return @dr if @dr

      @dr = Regexp.new(self.attributes.dialcode_expression) unless self.attributes.dialcode_expression.blank?
    end

    def dialcode_prefix_regexp
      return @dpr if @dpr

      @dpr = Regexp.new(self.attributes.dialcode_prefix_expression) unless self.attributes.dialcode_prefix_expression.blank?
    end

    def ani_regexp
      return @ar if @ar

      @ar = Regexp.new(self.attributes.ani_expression) unless self.attributes.ani_expression.blank?
    end

    def ani_prefix_regexp
      return @apr if @apr

      @apr = Regexp.new(self.attributes.ani_prefix_expression) unless self.attributes.ani_prefix_expression.blank?
    end

  end
end