module RouteNGNClient
  class OutboundEndPointGroup < RemoteModel

    set_resource_attributes({
                              :path_base => 'metallic/outbound_end_point_groups',
                              :collection_name => 'outbound_end_point_groups',
                              :per_page => 50,
                              :request_options => {
                              }
                            })

    belongs_to :carrier
    has_many :end_points, :outbound_rates

    def prioritized_end_points
      eps_with_priority = self.end_points.select { |ep| !ep.attributes.priority.nil? }
      eps_with_priority.shuffle!
      eps_without_priority = self.end_points.select { |ep| ep.attributes.priority.nil? }
      eps_without_priority.shuffle!

      eps_with_priority.sort! { |x,y| x.attributes.priority <=> y.attributes.priority }
      eps_without_priority.sort! { |x,y| x.random_priority <=> y.random_priority }

      eps = (eps_with_priority + eps_without_priority)
      self.attributes.max_contact_end_points ? eps[0, self.attributes.max_contact_end_points] : eps
    end

    def dialcode_regexp
      Regexp.new(self.attributes.dialcode_expression) unless self.attributes.dialcode_expression.blank?
    end

    def build_uri_parts(options = {})
      {
        :uri_user_params => self.s_to_h(self.attributes.uri_user_params ? self.attributes.uri_user_params.strip : ''),
        :uri_params => self.s_to_h(self.attributes.uri_params ? self.attributes.uri_params.strip : '')
      }
    end

    protected

      def s_to_h(s)
        h = {}
        m = s.match /^;*(.*);*$/

        h = Hash[m[1].split(';').map! {|p| p.split('=')}] if m[1]

        h
      end

  end
end