module RouteNGNClient
  class Record < RemoteModel

    set_resource_attributes({
                              :collection_name => 'records',
                              :per_page => 25
                            })

    belongs_to :account
    belongs_to :inbound_end_point_group

    def initialize(attributes = {})
      super
    end

    def create!(attributes = {}, options = {})
      options = options.merge({ :account_id => (self.account_id || (self.account.id if self.account)) })

      attrs = attributes.merge({
                :call_detail_record => (self.call_detail_record || self.call_detail_record_hash),
                :inbound_end_point_group_id => (self.inbound_end_point_group_id || (self.inbound_end_point_group.id if self.inbound_end_point_group)),
                :requested_at_float => (self.requested_at_float || (self.request.requested_at_float if self.request)),
                :response_code => (self.response_code || (self.final_response.response_code if self.final_response)),
                :logs => self.logs
              })

      self.class.create!(attrs, options)
    end

    def call_detail_record
      self.attributes.call_detail_record
    end

    def call_detail_record_hash
      self.class.values_to_hash(self.to_comma_headers, self.to_comma)
    end

    def routes_csvs(delimiter = ';')
      (self.routes || []).map { |r| CSV.generate_line(r.to_comma, :force_quotes => true, :col_sep => delimiter, :quote_char => '\'').strip! }
    end

    def routes_string(delimiter = '|')
      self.routes_csvs.join(delimiter)
    end

    def append_log(s, options = {})
      append_timestamp = options.has_key?(:append_timestamp) ? options[:append_timestamp] : true

      self.attributes.logs ||= []

      if append_timestamp
        age = self.attributes.request ? self.attributes.request.age.round(5) : nil
        self.attributes.logs << "#{age}: #{s}"
      else
        self.attributes.logs << "#{s}"
      end
    end

    def set_request(r)
      return unless r

      self.attributes.request = r
      self.requested_at_float = r.requested_at_float
      self.attributes.sip_request = r.sip_request
      self.append_log "Processing request: #{(r.sip_request.blank? || r.sip_request.method.blank?) ? 'Unknown' : r.sip_request.method} from #{r.peer_ip.blank? ? 'Unknown' : r.peer_ip}"
      self.append_log "#{r.sip_request.data if r.sip_request}"
    end

    def set_inbound_end_point_group(iepg)
      return unless iepg

      self.append_log "Using inbound_end_point_group:"
      self.append_log iepg.to_log, :append_timestamp => false
      self.attributes.inbound_end_point_group = iepg
      self.attributes.account = iepg.account if iepg
    end

    def set_inbound_rate(rate)
      return unless rate

      self.append_log "Using inbound_rate:"
      self.append_log rate.to_log, :append_timestamp => false
      self.attributes.inbound_rate = rate
    end

    def set_origination_code(code)
      return unless code

      self.append_log "Using origination_code:"
      self.append_log code.to_log, :append_timestamp => false
      self.attributes.origination_code = code
    end

    def set_destination_code(code)
      return unless code

      self.append_log "Using destination_code:"
      self.append_log code.to_log, :append_timestamp => false
      self.attributes.destination_code = code
    end

    def set_rate_type(type)
      return unless type

      self.append_log "Using rate_type: #{type}"
      self.attributes.rate_type = type
    end

    def add_route(route)
      return unless route

      self.append_log "Added route:"
      self.append_log route.to_log, :append_timestamp => false
      self.attributes.routes ||= []
      self.attributes.routes << route
    end

    def add_route_table(route_table)
      return unless route_table

      self.append_log "Using route_table:"
      self.append_log route_table.to_log, :append_timestamp => false
      self.attributes.route_tables ||= []
      self.attributes.route_tables << route_table
    end

    def final_route_table
      self.attributes.route_tables.blank? ? nil : self.attributes.route_tables.last
    end

    def add_response(r)
      return unless r

      self.attributes.responses ||= []
      self.attributes.responses << r
      self.append_log "Sending response code: #{(r.sip_response.blank? || r.sip_response.status_code.blank?) ? 'Unknown' : r.sip_response.status_code}"
      self.append_log "#{r.sip_response.data if r.sip_response}"
    end

    def final_response
      self.attributes.responses.blank? ? nil : self.attributes.responses.last
    end

    def request
      self.attributes.request
    end

    def sip_request
      self.attributes.sip_request
    end

    def origination_code
      self.attributes.origination_code
    end

    def destination_code
      self.attributes.destination_code
    end

    def inbound_rate
      self.attributes.inbound_rate
    end

    def sip_redirector_version
      self.attributes.sip_redirector_version
    end

    def cache_prep!
      attrs = Hashie::Mash.new
      attrs.call_detail_record = self.call_detail_record_hash
      attrs.account_id = self.account_id || (self.account.id if self.account)
      attrs.inbound_end_point_group_id = self.inbound_end_point_group.id if self.inbound_end_point_group
      attrs.requested_at_float = self.requested_at_float || (self.request.requested_at_float if self.request)
      attrs.response_code = self.final_response.response_code if self.final_response
      attrs.logs = self.logs
      self.attributes = attrs
    end

    def as_cdr(delimiter = ',')
      CSV.generate_line self.to_comma, :force_quotes => true, :col_sep => delimiter, :row_sep => nil
    end

    def self.values_to_hash(headers, values)
      values.each_with_index {| v,i| headers[i] = [headers[i].to_sym, v] }
      Hash[headers]
    end

    comma do
      request :requested_at_iso => 'requested_at_iso'
      request :requested_at_float => 'requested_at_float'
      request :server_hostname => 'server'
      request :peer_ip => 'peer_ip'
      request :peer_port => 'peer_port'
      request :sip_request_method => 'request_method'
      request :uuid => 'request_uuid'
      request :worker_id => 'worker_id'
      sip_request :prefix => 'request_prefix'
      sip_request :dialcode => 'request_dialcode'
      sip_request :call_id => 'request_call_id'
      sip_request :npdi => 'request_npdi'
      sip_request :rn => 'request_rn'
      sip_request :ani => 'request_ani'
      origination_code :dialcode => 'origination_dialcode'
      origination_code :local => 'origination_local'
      origination_code :city => 'origination_city'
      origination_code :state => 'origination_state'
      origination_code :country => 'origination_country'
      origination_code :lata => 'origination_lata'
      origination_code :ocn => 'origination_ocn'
      destination_code :dialcode => 'destination_dialcode'
      destination_code :local => 'destination_local'
      destination_code :city => 'destination_city'
      destination_code :state => 'destination_state'
      destination_code :country => 'destination_country'
      destination_code :lata => 'destination_lata'
      destination_code :ocn => 'destination_ocn'
      final_route_table :name => 'route_table_name'
      inbound_end_point_group :id => 'inbound_end_point_group_id'
      inbound_end_point_group :name => 'inbound_end_point_group_name'
      inbound_rate :price => 'inbound_rate_price'
      inbound_rate :type => 'inbound_rate_type'
      routes_string 'routes'  #"routes #{Route.to_comma_headers.join(';')}|route2|route3..."
      final_response :elapsed_time => 'elapsed_time'
      final_response :response_code => 'response_code'
      final_response :internal_response_code => 'internal_response_code'
      account :id => 'account_id'
      sip_redirector_version 'sip_redirector_version'
    end

  end
end