module RouteNGNClient
  class URIMatcher < Model
    include Comparable

    HEADER_ORDER = %(contact from request)

    belongs_to :allow_list

    def <=>(other)
      if @attributes.header == other.attributes.header
        case (@attributes.weight <=> other.attributes.weight)
        when 0
          @attributes.id.to_s <=> other.attributes.id.to_s
        when 1 #self.weight is higher
          -1
        when -1 #self.weight is lower
          1
        else
          0
        end
      else
        return 0 if !HEADER_ORDER.include?(@attributes.header) && !HEADER_ORDER.include?(other.attributes.header)
        return -1 if !HEADER_ORDER.include?(@attributes.header)
        return 1 if !HEADER_ORDER.include?(other.attributes.header)
        return (HEADER_ORDER.index(@attributes.header) <=> HEADER_ORDER.index(other.attributes.header))
      end
    end

    def weight
      @weight ||= [@attributes.prefix, @attributes.user_params, @attributes.uri_params].select { |x| !x.blank? }.length
    end

    def matches?(args = {})
      return false if !self.matches_header?(args[:header])
      return false if !@attributes.prefix.blank? && !self.matches_prefix?(args[:prefix])
      return false if !@attributes.user_params.blank? && !self.matches_user_params?(args[:user_params])
      return false if !@attributes.uri_params.blank? && !self.matches_uri_params?(args[:uri_params])
      return true
    end

    def matches_header?(arg)
      !@attributes.header.blank? && !arg.blank? && @attributes.header == arg
    end

    def matches_prefix?(arg)
      @attributes.prefix == arg
    end

    def matches_user_params?(arg)
      if arg.is_a?(String)
        @attributes.user_params == arg
      elsif arg.is_a?(Array)
        arg.include?(@attributes.user_params)
      else
        false
      end
    end

    def matches_uri_params?(arg)
      if arg.is_a?(String)
        @attributes.uri_params == arg
      elsif arg.is_a?(Array)
        arg.include?(@attributes.uri_params)
      else
        false
      end
    end

  end

  class UriMatcher < URIMatcher;end
end